@@ -6,11 +6,35 @@ import atexit
 import zipfile
 import tempfile
 import textwrap
+import functools
 from subprocess import check_call
+import pkg_resources
 
 def dedent(text):
     return textwrap.dedent(text.lstrip("\n"))
 
+def maintain_cwd(f):
+    @functools.wraps(f)
+    def maintain_cwd_helper(*args, **kwargs):
+        orig_dir = os.getcwd()
+        try:
+            return f(*args, **kwargs)
+        finally:
+            os.chdir(orig_dir)
+    return maintain_cwd_helper
+
+def egg_to_package(file):
+    """ Extracts the package name from an egg.
+
+        >>> egg_to_package("PyYAML-3.10-py2.7-macosx-10.7-x86_64.egg")
+        ('PyYAML', '3.10-py2.7-macosx-10.7-x86_64.egg')
+        >>> egg_to_package("python_ldap-2.3.9-py2.7-macosx-10.3-fat.egg")
+        ('python-ldap', '2.3.9-py2.7-macosx-10.3-fat.egg')
+    """
+    dist = pkg_resources.Distribution.from_location(None, file)
+    name = dist.project_name
+    return (name, file[len(name)+1:])
+
 def file_to_package(file, basedir=None):
     """ Returns the package name for a given file.
 
@@ -19,6 +43,8 @@ def file_to_package(file, basedir=None):
         >>> file_to_package("foo-bar-1.2.tgz")
         ('foo-bar', '1.2.tgz')
         >>> """
+    if os.path.splitext(file)[1].lower() == ".egg":
+        return egg_to_package(file)
     split = file.rsplit("-", 1)
     if len(split) != 2:
         msg = "unexpected file name: %r " %(file, )
@@ -27,12 +53,7 @@ def file_to_package(file, basedir=None):
             msg += "; found in directory: %r" %(basedir)
         msg += ")"
         raise ValueError(msg)
-    # Note: for various reasions (which I don't 100% remember right now) we
-    # need to replace '-' in the version string with '_'. I think this has to
-    # do with the way we export the list of files from the PIP manifest, then
-    # read them back in somewhere else. It would be cool to fix this at some
-    # point.
-    return (split[0], split[1].replace("_", "-"))
+    return (split[0], pkg_resources.safe_name(split[1]))
 
 def dir2pi(argv=sys.argv):
     if len(argv) != 2:
@@ -62,7 +83,7 @@ def dir2pi(argv=sys.argv):
                 packages/simple/foo/index.html
                 packages/simple/foo/foo-1.2.tar.gz
         """)
-        sys.exit(1)
+        return 1
     pkgdir = argv[1]
     if not os.path.isdir(pkgdir):
         raise ValueError("no such directory: %r" %(pkgdir, ))
@@ -90,8 +111,9 @@ def dir2pi(argv=sys.argv):
             pkg_new_basename_html = cgi.escape(pkg_new_basename)
             fp.write("<a href='%s'>%s</a><br />\n"
                      %(pkg_new_basename_html, pkg_new_basename_html))
+    return 0
 
-
+@maintain_cwd
 def pip2tgz(argv=sys.argv):
     if len(argv) < 3:
         print dedent("""
@@ -107,7 +129,7 @@ def pip2tgz(argv=sys.argv):
                 
                 $ pip2tgz /var/www/packages/ -r requirements.txt foo==1.2 baz/
         """)
-        sys.exit(1)
+        return 1
 
     outdir = os.path.abspath(argv[1])
     if not os.path.exists(outdir):
@@ -152,6 +174,7 @@ def pip2tgz(argv=sys.argv):
     os.chdir(outdir)
     shutil.rmtree(tempdir)
     print "%s .tar.gz saved to %r" %(num_pakages, argv[1])
+    return 0
 
 def pip2pi(argv=sys.argv):
     if len(argv) < 3:
@@ -173,7 +196,7 @@ def pip2pi(argv=sys.argv):
 
                 $ pip2pi ~/Sites/packages/ foo==1.2
         """)
-        sys.exit(1)
+        return 1
 
     target = argv[1]
     pip_packages = argv[2:]
@@ -185,8 +208,15 @@ def pip2pi(argv=sys.argv):
         is_remote = False
         working_dir = os.path.abspath(target)
 
-    pip2tgz([argv[0], working_dir] + pip_packages)
-    dir2pi([argv[0], working_dir])
+    res = pip2tgz([argv[0], working_dir] + pip_packages)
+    if res:
+        print "pip2tgz returned an error; aborting."
+        return res
+
+    res = dir2pi([argv[0], working_dir])
+    if res:
+        print "dir2pi returned an error; aborting."
+        return res
 
     if is_remote:
         print "copying temporary index at %r to %r..." %(working_dir, target)
@@ -195,3 +225,4 @@ def pip2pi(argv=sys.argv):
             "--recursive", "--progress", "--links",
             working_dir + "/", target + "/",
         ])
+    return 0
